
:host {
    display: flex;
    flex-direction: column;
    flex-grow: 1;
    padding: 16px;
}

.cache-status-container {
    width: 100%;

    .status-card {
        width: 100%;
        margin-bottom: 16px;

        &:last-child {
            margin-bottom: 0;
        }
    }
}